/*
 * actions.h
 */

#ifndef ACTIONS_H
#define ACTIONS_H

void quit_app();
void set_values();
void save_datas();
void add_alarm();
void edit_entry();
void switch_onoff();
void remove_alarm();

#endif
