/*
 * main variables for inclusion
 */

#ifndef VARIABLES_H
#define VARIABLES_H

#include "defines.h"

int    backlight;
int    switch_authorized;
int    h12;
int    time_mode;
int    use_leds;
int    use_locale;
int    showcal;
int    calalrms;
char  *style_name;
char  *style_dir;
char  *config_file;
char  *light_color;
char  *command;
char  *msgcmd;
Alarm *alarms;

#endif
