/*
 * Constants and types definitions
 */

#ifndef DEFINES_H
#define DEFINES_H

#define DEFAULT_CONFIGDIR ".wmclockmoncal"

#define MAXSTRLEN 1024

#define WIN_WIDTH  300
#define WIN_HEIGHT 240

#ifndef TRUE
# define TRUE  1
# define FALSE 0
#endif


#endif
